/*
			Copyright (c) 1989,1990,1992 by
			Northeast Parallel Architectures Center
			All Rights Reserved
	
			This program is written by Gang Cheng(gcheng@npac.syr.edu)
			Date: 10/12/93
	
*/
#include <stdio.h>
#include <avs/avs.h>
#include <avs/field.h>
#include <avs/geom.h>

/******************************************************************************/

/* 
 * This is a C example to create a geometry object.  In this example,
 * the "simulation" program flow of control is used.  Instead of providing
 * a compute module function that is called whenver a parameter or input
 * has changed, this module can determine when it wants to provide new
 * data to the network.  Many existing applications will fit into this 
 * model much more easily than the "compute function" model.
 *   
 */  

/*  
 * The routine "tracker" is the description routine.  It provides
 * AVS some necessary information such as: name, input and output ports, 
 * parameters etc.
 */
tracker()
{
	int out_port;		/* temporary to hold the port number */
	int parm;		/* temporary to hold the parm number */

	/* Set the module name and type */
	AVSset_module_name("tracker", MODULE_DATA);

	/* There are no input ports for this module */

	/* Create an output port for the result */
	out_port = AVScreate_output_port("Output Geometry", "geom");

	/* Add parameter: an enable/disable toggle for the scope */
	parm = AVSadd_parameter("sleep", "boolean", 1, 0, 1);
	/* Add parameter: number of cycles before putting self to sleep */
	parm = AVSadd_parameter("cycles", "integer", 0, 0, 10000);

	/* There is no compute function for this module */
}

#define MAXV 200
#define PERFRAME 1
#define FACTOR 100.

FILE *fid=NULL; 
static float scale[3];
int radius = 1;

main(argc,argv)
int	argc;
char	*argv[];
{
   int tracker();
   int count = MAXV;
   int sleep = 1;
   int num = MAXV;
   int cycles = 0,cycle_count=0;
   GEOMobj *obj = NULL;
   GEOMedit_list output = NULL;
   int i;
   float ext[6];
   float *centers, *colors, *radii;

/*
 * We initialize this module by passing a pointer to our description
 * function.  The description function is defined below.  It defines our
 * inputs, outputs and parameters.
 * When AVS loads this module, this program is invoked, the description
 * function is called to identify the module (and get the colors on the
 * palette), then the program exits.  When the module is dropped onto
 * the palette, the program is invoked, the description function
 * is called and this function will return.
 */
   AVScorout_init(argc,argv,tracker);

   AVScorout_set_sync(1);

   while(1) {
/*
 * If our "sleep" parameter is false, we mark ourselves as having changed.
 * This will cause the subsequent call to AVScorout_wait() to return 
 * right away.  Otherwise, AVScorout_wait will pause until inputs or parameters
 * are changed.
 */
      if (!sleep) AVScorout_mark_changed();

      AVScorout_wait();

 /*
 * We get the value of the "sleep" parameter here.  This might cause us to
 * do one extra iteration of the loop but since this is a string art demo
 * it won't matter.  Note that the parameters to AVScorout_input vary
 * based on the number of inputs and parameters that you have.
 */
      AVScorout_input(&sleep,&cycles);
/*
 * For each frame, we compute "PERFRAME" number of spheres to add to our
 * object.  We re-use the same object adding sphere until we get to the
 * threshold, then destroy the object and start over again.
 */
      centers = (float *)malloc((unsigned)(PERFRAME*sizeof(float)*3));
      colors = (float *)malloc((unsigned)(PERFRAME*sizeof(float)*3));
      radii = (float *)malloc((unsigned)(PERFRAME*sizeof(float)));

      if (count >= num) {

	if (fid != NULL) fclose(fid);
	fid = fopen("/home/mars/gcheng/track/test.sphere","r");
	if (fid == NULL) fprintf(stderr,"open file error \n");
	fscanf(fid,"%f%f%f%f%f%f",&ext[0],&ext[2],&ext[4],&ext[1],&ext[3],&ext[5]);
	scale[0] = ext[0]; /* x, y, z coordinates scale factor */
	scale[1] = ext[2];
	scale[2] = ext[4];
	ext[1] -= ext[0]; ext[0] = 0;
	ext[3] -= ext[2]; ext[2] = 0;
	ext[5] -= ext[4]; ext[4] = 0;
/*	for (i=0; i<6; i++)
	  ext[i] /= FACTOR; */
	fscanf(fid,"%d",&num);	fprintf(stderr,"num = %d\n",num);
	count = 0; radius = 0.5*ext[1]/(float) num;
	if (obj) GEOMdestroy_obj(obj);
	obj = GEOMcreate_obj(GEOM_SPHERE,ext);
      }
      
      next(centers,radii,colors);
      GEOMadd_vertices(obj,centers,PERFRAME,GEOM_DONT_COPY_DATA);
      GEOMadd_radii(obj,radii,PERFRAME,GEOM_DONT_COPY_DATA);
      GEOMadd_float_colors(obj,colors,PERFRAME,GEOM_DONT_COPY_DATA);

      /* GEOMadd_disjoint_line(obj,verts,colors,2,GEOM_COPY_DATA); */
      GEOMauto_transform(obj);
      count += PERFRAME;

/*
 * Once we have an object that we want to display, we create a new
 * edit list.  An edit list contains a list of the changes that we want to
 * make to the scene on any given frame.  We clear out the list (freeing
 * the one from the previous frame), then tell AVS to replace the geometry
 * of an object named 'tracker' with the new object.  Then we take the 
 * resulting edit list and output it to AVS.  Note that the parameters
 * to AVScorout_output vary based on the number of outputs that you have.
 */
      output = GEOMinit_edit_list(output);
      GEOMedit_geometry(output,"tracker",obj);
      AVScorout_output(output);
      if (cycles && (cycle_count++ > cycles))
	AVSmodify_parameter("sleep",AVS_VALUE,1,0,0);
      fprintf(stderr,"finish one frame. Total = %d points\n",count);
   }
}

next(centers,radii,colors)
float *centers,*radii,*colors;
{
   int i;
   for (i = 0; i < PERFRAME; i++) {
     fscanf(fid,"%f%f%f%f%f%f%f",&centers[i*3],
	    &centers[i*3+1],&centers[i*3+2],
	    &radii[i],&colors[i*3],&colors[i*3+1],&colors[i*3+2]);
     centers[i*3] -= scale[0];
     centers[i*3+1] -= scale[1];
     centers[i*3+2] -= scale[2];
     radii[i] = radius;
   } 
}
